/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: atomic_amd64_gcc.c 56 2008-08-26 18:45:22Z river $ */

#include	"swatomic.h"

/*
 * sw_atomic_inc_
 */
void
sw_atomic_inc(volatile sw_atomic_t *x)
{
	asm("lock; incq (%0)" : : "r" (x));
}

sw_atomic_t
sw_atomic_inc_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	asm volatile(
		"movq (%1), %0\n"
		"1:\n"
		"  movq %0, %%rbx\n"
		"  incq %%rbx\n"
		"  lock\n"
		"  cmpxchgq %%rbx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x) : "%rbx");
	return oldval;
}

sw_atomic_t
sw_atomic_inc_nv(volatile sw_atomic_t *x)
{
	sw_atomic_t newval;
	asm volatile(
		"movq (%1), %%rax\n"
		"1:\n"
		"  movq %%rax, %0\n"
		"  incq %0\n"
		"  lock\n"
		"  cmpxchgq %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x) : "%rax");
	return newval;
}

/*
 * sw_atomic_dec_
 */
void
sw_atomic_dec(volatile sw_atomic_t *x)
{
	asm("lock; decq (%0)" : : "r" (x));
}

sw_atomic_t
sw_atomic_dec_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	asm volatile(
		"movq (%1), %0\n"
		"1:\n"
		"  movq %0, %%rbx\n"
		"  decq %%rbx\n"
		"  lock\n"
		"  cmpxchgq %%rbx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x) : "%rbx");
	return oldval;
}

sw_atomic_t
sw_atomic_dec_nv(volatile sw_atomic_t *x)
{
	sw_atomic_t newval;
	asm volatile(
		"movq (%1), %%rax\n"
		"1:\n"
		"  movq %%rax, %0\n"
		"  decq %0\n"
		"  lock\n"
		"  cmpxchgq %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x) : "%rax");
	return newval;
}

/* 
 * sw_atomic_add_
 */
void
sw_atomic_add(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	asm("lock; addq %0, (%1)" : : "r" (inc), "r" (x));
}

sw_atomic_t
sw_atomic_add_ov(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t oldval;
	asm volatile(
		"movq (%1), %0\n"
		"1:\n"
		"  movq %0, %%rbx\n"
		"  addq %2, %%rbx\n"
		"  lock\n"
		"  cmpxchgq %%rbx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x), "c" (inc): "%rbx");
	return oldval;
}

sw_atomic_t
sw_atomic_add_nv(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t newval;
	asm volatile(
		"movq (%1), %%rax\n"
		"1:\n"
		"  movq %%rax, %0\n"
		"  addq %2, %0\n"
		"  lock\n"
		"  cmpxchgq %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x), "c" (inc) : "%rax");
	return newval;
}

/* 
 * sw_atomic_sub_
 */
void
sw_atomic_sub(volatile sw_atomic_t *x, sw_atomic_t dec)
{
	asm("lock; subq %0, (%1)" : : "r" (dec), "r" (x));
}

sw_atomic_t
sw_atomic_sub_ov(volatile sw_atomic_t *x, sw_atomic_t sub)
{
	sw_atomic_t oldval;
	asm volatile(
		"movq (%1), %0\n"
		"1:\n"
		"  movq %0, %%rbx\n"
		"  subq %2, %%rbx\n"
		"  lock\n"
		"  cmpxchgq %%rbx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x), "c" (sub): "%rbx");
	return oldval;
}

sw_atomic_t
sw_atomic_sub_nv(volatile sw_atomic_t *x, sw_atomic_t sub)
{
	sw_atomic_t newval;
	asm volatile(
		"movq (%1), %%rax\n"
		"1:\n"
		"  movq %%rax, %0\n"
		"  subq %2, %0\n"
		"  lock\n"
		"  cmpxchgq %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x), "c" (sub) : "%rax");
	return newval;
}

/*
 * sw_atomic_swap
 */
void *
sw_atomic_swap_ptr_ov(void *volatile *valptr, void *val)
{
	asm volatile(	
		"lock; xchgq %0, (%1)\n"
		: "=r" (val) : "r" (valptr), "0" (val) );
	return val;
}

void
sw_atomic_swap_ptr(void *volatile *valptr, void *newval)
{
	sw_atomic_swap_ptr_ov(valptr, newval);
}
