/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: atomic_i386_gcc.c 56 2008-08-26 18:45:22Z river $ */

#include	"swatomic.h"

/*
 * sw_atomic_inc_
 */
void
sw_atomic_inc(volatile sw_atomic_t *x)
{
	asm("lock; incl (%0)" : : "r" (x));
}

sw_atomic_t
sw_atomic_inc_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	asm volatile(
		"movl (%1), %0\n"
		"1:\n"
		"  movl %0, %%ebx\n"
		"  incl %%ebx\n"
		"  lock\n"
		"  cmpxchgl %%ebx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x) : "%ebx");
	return oldval;
}

sw_atomic_t
sw_atomic_inc_nv(volatile sw_atomic_t *x)
{
	sw_atomic_t newval;
	asm volatile(
		"movl (%1), %%eax\n"
		"1:\n"
		"  movl %%eax, %0\n"
		"  incl %0\n"
		"  lock\n"
		"  cmpxchgl %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x) : "%eax");
	return newval;
}

/*
 * sw_atomic_dec_
 */
void
sw_atomic_dec(volatile sw_atomic_t *x)
{
	asm("lock; decl (%0)" : : "r" (x));
}

sw_atomic_t
sw_atomic_dec_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	asm volatile(
		"movl (%1), %0\n"
		"1:\n"
		"  movl %0, %%ebx\n"
		"  decl %%ebx\n"
		"  lock\n"
		"  cmpxchgl %%ebx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x) : "%ebx");
	return oldval;
}

sw_atomic_t
sw_atomic_dec_nv(volatile sw_atomic_t *x)
{
	sw_atomic_t newval;
	asm volatile(
		"movl (%1), %%eax\n"
		"1:\n"
		"  movl %%eax, %0\n"
		"  decl %0\n"
		"  lock\n"
		"  cmpxchgl %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x) : "%eax");
	return newval;
}

/* 
 * sw_atomic_add_
 */
void
sw_atomic_add(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	asm("lock; addl %0, (%1)" : : "r" (inc), "r" (x));
}

sw_atomic_t
sw_atomic_add_ov(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t oldval;
	asm volatile(
		"movl (%1), %0\n"
		"1:\n"
		"  movl %0, %%ebx\n"
		"  addl %2, %%ebx\n"
		"  lock\n"
		"  cmpxchgl %%ebx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x), "c" (inc): "%ebx");
	return oldval;
}

sw_atomic_t
sw_atomic_add_nv(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t newval;
	asm volatile(
		"movl (%1), %%eax\n"
		"1:\n"
		"  movl %%eax, %0\n"
		"  addl %2, %0\n"
		"  lock\n"
		"  cmpxchgl %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x), "c" (inc) : "%eax");
	return newval;
}

/* 
 * sw_atomic_sub_
 */
void
sw_atomic_sub(volatile sw_atomic_t *x, sw_atomic_t dec)
{
	asm("lock; subl %0, (%1)" : : "r" (dec), "r" (x));
}

sw_atomic_t
sw_atomic_sub_ov(volatile sw_atomic_t *x, sw_atomic_t sub)
{
	sw_atomic_t oldval;
	asm volatile(
		"movl (%1), %0\n"
		"1:\n"
		"  movl %0, %%ebx\n"
		"  subl %2, %%ebx\n"
		"  lock\n"
		"  cmpxchgl %%ebx, (%1)\n"
		"  jne 1b\n"
		: "=a" (oldval) : "d" (x), "c" (sub): "%ebx");
	return oldval;
}

sw_atomic_t
sw_atomic_sub_nv(volatile sw_atomic_t *x, sw_atomic_t sub)
{
	sw_atomic_t newval;
	asm volatile(
		"movl (%1), %%eax\n"
		"1:\n"
		"  movl %%eax, %0\n"
		"  subl %2, %0\n"
		"  lock\n"
		"  cmpxchgl %0, (%1)\n"
		"  jne 1b\n"
		: "=b" (newval) : "d" (x), "c" (sub) : "%eax");
	return newval;
}

/*
 * sw_atomic_swap
 */
void *
sw_atomic_swap_ptr_ov(void *volatile *valptr, void *val)
{
	asm volatile(	
		"lock; xchgl %0, (%1)\n"
		: "=r" (val) : "r" (valptr), "0" (val) );
	return val;
}

void
sw_atomic_swap_ptr(void *volatile *valptr, void *newval)
{
	sw_atomic_swap_ptr_ov(valptr, newval);
}
