/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: atomic_solaris.c 52 2008-08-25 21:21:08Z river $ */

#include	<pthread.h>

#include	"swatomic.h"

static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;

/*
 * sw_atomic_inc_
 */
void
sw_atomic_inc(volatile sw_atomic_t *x)
{
	pthread_mutex_lock(&mtx);
	(*x)++;
	pthread_mutex_unlock(&mtx);
}

sw_atomic_t
sw_atomic_inc_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	pthread_mutex_lock(&mtx);
	oldval = (*x)++;
	pthread_mutex_unlock(&mtx);
	return oldval;
}

sw_atomic_t
sw_atomic_inc_nv(volatile sw_atomic_t *x)
{
	sw_atomic_t newval;
	pthread_mutex_lock(&mtx);
	newval = ++(*x);
	pthread_mutex_unlock(&mtx);
	return newval;
}

/*
 * sw_atomic_dec_
 */
void
sw_atomic_dec(volatile sw_atomic_t *x)
{
	pthread_mutex_lock(&mtx);
	(*x)--;
	pthread_mutex_unlock(&mtx);
}

sw_atomic_t
sw_atomic_dec_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	pthread_mutex_lock(&mtx);
	oldval = (*x)--;
	pthread_mutex_unlock(&mtx);
	return oldval;
}

sw_atomic_t
sw_atomic_dec_nv(volatile sw_atomic_t *x)
{
	sw_atomic_t newval;
	pthread_mutex_lock(&mtx);
	newval = --(*x);
	pthread_mutex_unlock(&mtx);
	return newval;
}

/*
 * sw_atomic_add
 */
void
sw_atomic_add(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	pthread_mutex_lock(&mtx);
	(*x) += inc;
	pthread_mutex_unlock(&mtx);
}

sw_atomic_t
sw_atomic_add_ov(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t oldval;
	pthread_mutex_lock(&mtx);
	oldval = *x;
	(*x) += inc;
	pthread_mutex_unlock(&mtx);
	return oldval;
}

sw_atomic_t
sw_atomic_add_nv(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t newval;
	pthread_mutex_lock(&mtx);
	*x += inc;
	newval = *x;
	pthread_mutex_unlock(&mtx);
	return newval;
}

/*
 * sw_atomic_sub
 */
void
sw_atomic_sub(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	pthread_mutex_lock(&mtx);
	(*x) -= inc;
	pthread_mutex_unlock(&mtx);
}

sw_atomic_t
sw_atomic_sub_ov(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t oldval;
	pthread_mutex_lock(&mtx);
	oldval = *x;
	(*x) -= inc;
	pthread_mutex_unlock(&mtx);
	return oldval;
}

sw_atomic_t
sw_atomic_sub_nv(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t newval;
	pthread_mutex_lock(&mtx);
	*x -= inc;
	newval = *x;
	pthread_mutex_unlock(&mtx);
	return newval;
}

void
sw_atomic_swap_ptr(void *volatile *valptr, void *newval)
{
	sw_atomic_swap_ptr(valptr, newval);
}

void *
sw_atomic_swap_ptr_ov(void *volatile *valptr, void *newval)
{
	void *oldval;
	pthread_mutex_lock(&mtx);
	oldval = *valptr;
	*valptr = newval;
	pthread_mutex_unlock(&mtx);
	return oldval;
}
