/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: atomic_solaris.c 56 2008-08-26 18:45:22Z river $ */

#include	<atomic.h>

#include	"swatomic.h"

/*
 * sw_atomic_inc_
 */
void
sw_atomic_inc(volatile sw_atomic_t *x)
{
	atomic_inc_ulong(x);
}

sw_atomic_t
sw_atomic_inc_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	do {
		oldval = *x;
	} while (atomic_cas_ulong(x, oldval, oldval + 1) != oldval);
	return oldval;
}

sw_atomic_t
sw_atomic_inc_nv(volatile sw_atomic_t *x)
{
	return atomic_inc_ulong_nv(x);
}

/*
 * sw_atomic_dec_
 */
void
sw_atomic_dec(volatile sw_atomic_t *x)
{
	atomic_dec_ulong(x);
}

sw_atomic_t
sw_atomic_dec_ov(volatile sw_atomic_t *x)
{
	sw_atomic_t oldval;
	do {
		oldval = *x;
	} while (atomic_cas_ulong(x, oldval, oldval - 1) != oldval);
	return oldval;
}

sw_atomic_t
sw_atomic_dec_nv(volatile sw_atomic_t *x)
{
	return atomic_dec_ulong_nv(x);
}

/*
 * sw_atomic_add
 */
void
sw_atomic_add(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	atomic_add_long(x, inc);
}

sw_atomic_t
sw_atomic_add_ov(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t oldval;
	do {
		oldval = *x;
	} while (atomic_cas_ulong(x, oldval, oldval + inc) != oldval);
	return oldval;
}

sw_atomic_t
sw_atomic_add_nv(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	return atomic_add_long_nv(x, inc);
}

/*
 * sw_atomic_sub
 */
void
sw_atomic_sub(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	atomic_add_long(x, -(long)inc);
}

sw_atomic_t
sw_atomic_sub_ov(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	sw_atomic_t oldval;
	do {
		oldval = *x;
	} while (atomic_cas_ulong(x, oldval, oldval - inc) != oldval);
	return oldval;
}

sw_atomic_t
sw_atomic_sub_nv(volatile sw_atomic_t *x, sw_atomic_t inc)
{
	return atomic_add_long_nv(x, -(long)inc);
}

void
sw_atomic_swap_ptr(void *volatile *valptr, void *newval)
{
	atomic_swap_ptr(valptr, newval);
}

void *
sw_atomic_swap_ptr_ov(void *volatile *valptr, void *newval)
{
	return atomic_swap_ptr(valptr, newval);
}
