/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: config.h 62 2008-08-28 14:36:30Z river $ */

#ifndef CONFIG_H
#define CONFIG_H

#include	<sys/types.h>

#include	<string>
#include	<vector>
#include	<stdexcept>
#include	<map>
#include	<iostream>

#include	<boost/function.hpp>
#include	<boost/intrusive_ptr.hpp>

#include	"swatomic.h"

struct config_error : std::runtime_error {
	config_error(char const *what) : std::runtime_error(what) {}
};

struct conf_listener {
	std::string host;
	std::string port;
};

enum server_type {
	serv_unknown = 0,
	serv_sjs,
	serv_sjs_script_filename,
	serv_apache,
	serv_zeus
};

struct config {
	config();

	std::vector<conf_listener> listeners;
	std::vector<conf_listener> status_listeners;
	std::string sockdir;
	std::string docroot;
	std::string userdir;
	int nfiles;
	int max_procs;
	int max_procs_per_user;
	int max_q_per_user;
	int php_timeout;
	int server_timeout;
	int max_request_size;
	server_type servtype;
	bool log_request_errors;
	std::string fcgi_extn;
	std::string admin_contact;
	std::string pid_file;
	std::map<uid_t, int> per_user_max_procs;
	
	sw_atomic_t refcnt;
};
typedef boost::intrusive_ptr<config> configp;

void intrusive_ptr_add_ref(config *);
void intrusive_ptr_release(config *);

struct configuration_loader {
	configuration_loader();

	bool	load(std::string const &filename, config &newconf);

private:
	bool parse_config_line(std::string const &line, config &newconf);

	int lineno_;
	std::string file_;

	typedef std::map<std::string, std::string> confopts_t;
	typedef std::vector<std::string> fieldlist_t;
	typedef boost::function<bool (configuration_loader *,
					fieldlist_t &,
					confopts_t &,
					config &newconf)> confline_t;
	
	static std::map<std::string, confline_t> conflines;

	bool f_listen		(fieldlist_t &, confopts_t &, config &newconf);
	bool f_status_listen	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_sockdir		(fieldlist_t &, confopts_t &, config &newconf);
	bool f_docroot		(fieldlist_t &, confopts_t &, config &newconf);
	bool f_userdir		(fieldlist_t &, confopts_t &, config &newconf);
	bool f_max_procs	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_max_procs_per_user(fieldlist_t &, confopts_t &, config &newconf);
	bool f_max_q_per_user	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_server_type	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_server_timeout	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_php_timeout	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_max_request_size	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_log_request_errors(fieldlist_t &, confopts_t &, config &newconf);
	bool f_fcgi_extn	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_admin_contact	(fieldlist_t &, confopts_t &, config &newconf);
	bool f_pid_file		(fieldlist_t &, confopts_t &, config &newconf);
	bool f_nfiles		(fieldlist_t &, confopts_t &, config &newconf);
	
	bool _set_int(std::string const &help, fieldlist_t &, confopts_t &, int *where);
	bool _set_string(std::string const &help, fieldlist_t &, confopts_t &, std::string *where);	
};

void install_new_config(config *);
configp get_current_config();

#endif
