/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: error.cc 52 2008-08-25 21:21:08Z river $ */

#include	<fstream>
#include	<sstream>
#include	<iostream>

#include	<unistd.h>
#include	<netdb.h>

#include	"error.h"
#include	"util.h"
#include	"version.h"
#include	"config.h"
#include	"setup.h"

error::error(
	std::map<std::string, std::string> const &params,
	configp const &conf)
	: params_(params)
	, conf_(conf)
{
}

error::error()
{
}

error::~error()
{
}

std::string
error::format(std::string const &errstr)
{
	std::ifstream f(PREFIX "/share/switchboard/errors/general.html");
	std::string hostname(get_host_name());
	
	if (!f)
		return "error file not found";

	std::map<std::string, std::string> vars;
	vars["VERSION"] = SB_VERSION;
	vars["HOSTNAME"] = hostname;
	vars["ERROR"] = errstr;

	if (conf_->admin_contact.empty())
		vars["ADMIN"] = "address not specified";
	else
		vars["ADMIN"] = conf_->admin_contact;

	std::ostringstream strm;
	strm << f.rdbuf();
	f.close();


	for (std::map<std::string, std::string>::const_iterator
			it = params_.begin(), end = params_.end();
			it != end; ++it)
	{
		vars[it->first] = htmlescape(it->second);
	}

	std::string source = strm.str();
	std::string result;
	std::string::size_type n, p, last = 0;
	while ((n = source.find('%', last)) != std::string::npos) {
		if ((p = source.find('%', n + 1)) == std::string::npos)
			break;
		result.append(source.begin() + last, source.begin() + n);
		std::string var = source.substr(n + 1, (p - 1) - n);
		if (vars.find(var) == vars.end())
			result += source.substr(n, p - n + 1);
		else
			result += vars[var];
		last = p + 1;
	}
	result += source.substr(last);

	return result;
}
