/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: exception.h 42 2008-08-21 04:36:01Z river $ */

#ifndef EXCEPTION_H
#define EXCEPTION_H

#include	<stdexcept>
#include	<string>

#include	<netdb.h>

#include	<boost/format.hpp>

struct errno_exception : std::runtime_error {
	errno_exception(char const *what, int err = errno)
		: std::runtime_error(what)
		, err_(err) {
			what_ = str(boost::format("%s: %s")
				% what % std::strerror(err));
		}

	errno_exception(std::string const &what, int err = errno)
		: std::runtime_error(what)
		, err_(err) {
			what_ = str(boost::format("%s: %s")
				% what % std::strerror(err));
		}
	~errno_exception() throw() {}

	char const *what() const throw() {
		return what_.c_str();
	}

	int err() const throw() {
		return err_;
	}

private:
	std::string what_;
	int err_;
};

struct gai_exception : std::runtime_error {
	gai_exception(char const *what, int err)
		: std::runtime_error(what)
		, err_(err) {
			what_ = str(boost::format("%s: %s")
				% what % gai_strerror(err));
		}
	gai_exception(std::string const &what, int err)
		: std::runtime_error(what)
		, err_(err) {
			what_ = str(boost::format("%s: %s")
				% what % gai_strerror(err));
		}
	~gai_exception() throw() {}

	char const *what() const throw() {
		return what_.c_str();
	}

	int err() const throw() {
		return err_;
	}

private:
	std::string what_;
	int err_;
};

#endif	/* !EXCEPTION_H */
