/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: logger.cc 42 2008-08-21 04:36:01Z river $ */

#include	<ctime>
#include	<iostream>

#include	<syslog.h>

#include	<boost/format.hpp>

#include	"logger.h"

logger_t logger;

logger_t::logger_t()
	: target_(log_console)
	, severity_(sev_info)
{
}

void
logger_t::set_severity(log_severity sev)
{
	severity_ = sev;
}

void
logger_t::set_target(log_target targ)
{
	if (target_ != log_syslog && targ == log_syslog) 
		openlog("switchboard", LOG_PID, LOG_DAEMON);
	else if (target_ == log_syslog && targ != log_syslog)
		closelog();

	target_ = targ;
}

void
logger_t::debug(std::string const &s)
{
	_log(sev_debug, s);
}

void
logger_t::info(std::string const &s)
{
	_log(sev_info, s);
}

void
logger_t::warn(std::string const &s)
{
	_log(sev_warn, s);
}

void
logger_t::error(std::string const &s)
{
	_log(sev_error, s);
}

void
logger_t::_log(log_severity sev, std::string const &s)
{
	if (sev < severity_)
		return;

	std::time_t t = std::time(0);
	struct std::tm *tim = std::gmtime(&t);
	char tbuf[256];
	std::strftime(tbuf, sizeof(tbuf), "%Y-%m-%d %H:%M:%S", tim);

	std::string msg;

	if (target_ == log_syslog) {
		msg = str(boost::format("[%-5s] %s") % _sevstr(sev) % s);
	} else {
		msg = str(boost::format("[%-5s] %s: %s") % _sevstr(sev) % tbuf % s);
	}

	switch (target_) {
	case log_console:
		std::cerr << msg << '\n';
		break;
	case log_syslog:
		{
			int ssev = LOG_INFO;
			switch (sev) {
			case sev_error: 
				ssev = LOG_ERR;
				break;
			case sev_warn:
				ssev = LOG_WARNING;
				break;
			}

			syslog(ssev, "%s", msg.c_str());
		}
		break;
	}
}

std::string
logger_t::_sevstr(log_severity sev)
{
	switch (sev) {
	case sev_debug:
		return "debug";
	case sev_info:
		return "info";
	case sev_warn:
		return "warn";
	case sev_error:
		return "error";
	default:
		return "unknown";
	}
}
