/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: logger.h 42 2008-08-21 04:36:01Z river $ */

#ifndef LOGGER_H
#define LOGGER_H

#include	<string>
#include	<sstream>

enum log_target {
	log_syslog,
	log_console
};

enum log_severity {
	sev_debug = 0,
	sev_info,
	sev_warn,
	sev_error
};

struct logger_t {
	logger_t();

	void set_target(log_target);
	void set_severity(log_severity);

	void debug(std::string const &msg);
	template<typename T>
	void debug(T const &v) {
		std::ostringstream strm;
		strm << v;
		debug(strm.str());
	}

	void info(std::string const &msg);
	template<typename T>
	void info(T const &v) {
		std::ostringstream strm;
		strm << v;
		info(strm.str());
	}

	void warn(std::string const &msg);
	template<typename T>
	void warn(T const &v) {
		std::ostringstream strm;
		strm << v;
		warn(strm.str());
	}

	void error(std::string const &msg);
	template<typename T>
	void error(T const &v) {
		std::ostringstream strm;
		strm << v;
		error(strm.str());
	}

private:
	log_target target_;
	log_severity severity_;

	void _log(log_severity, std::string const &);
	std::string _sevstr(log_severity sev);
};

extern logger_t logger;

#endif	/* !LOGGER_H */
