/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: process.h 44 2008-08-21 04:46:56Z river $ */

#ifndef PROCESS_H
#define PROCESS_H

#include	<sys/types.h>

#include	<stdexcept>

#include	<boost/shared_ptr.hpp>
#include	<boost/noncopyable.hpp>

struct connect_failure : std::runtime_error {
	connect_failure(char const *error) : std::runtime_error(error) {}
};

struct creation_failure : std::runtime_error {
	creation_failure(char const *error) : std::runtime_error(error) {}
};

struct security_violation : creation_failure {
	security_violation(char const *error) : creation_failure(error) {}
};

struct process : boost::noncopyable {
	process(uid_t, gid_t, std::string const &bindpath, std::string const &proram);
	~process();

	int connect();

	uid_t uid() const { return uid_; }
	gid_t gid() const { return gid_; }
	pid_t pid() const { return pid_; }
	std::string const &program() const { return program_; }

private:
	std::string bindpath_;
	std::string program_;
	pid_t pid_;
	uid_t uid_;
	gid_t gid_;
};

typedef boost::shared_ptr<process> processp;

#endif
