/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: request_thread.h 52 2008-08-25 21:21:08Z river $ */

#ifndef REQUEST_THREAD_H
#define REQUEST_THREAD_H

#include	<stdexcept>
#include	<cerrno>
#include	<cstring>

#include	<pthread.h>

#include	<boost/format.hpp>

#include	"fcgi.h"
#include	"process.h"
#include	"config.h"

struct request_exception : std::runtime_error {
	request_exception(char const *what)
		: std::runtime_error(what) {}
};

struct request_thread {
	request_thread(int fd, configp const &);
	~request_thread();

	void start();
	bool start_request();

	static void accept_handler(int, configp const &);
	configp conf() const { return conf_; }
private:
	void handle_normal_request(fcgi::record &);
	void handle_normal_request_child();
	void handle_get_values(fcgi::record &);

	int fd_;
	int cfd_;
	int rid_;
	pthread_t tid_;

	std::vector<unsigned char> stdin_;
	std::vector<unsigned char> paramdata_;
	processp process_;
	std::map<std::string, std::string> params_;
	configp conf_;
};

#endif	/* !REQUEST_THREAD_H */
