/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: stats_thread.h 56 2008-08-26 18:45:22Z river $ */

#ifndef STATUS_THREAD_H
#define STATUS_THREAD_H

#include	<string>

#include	<netdb.h>
#include	<unistd.h>
#include	<pthread.h>

#include	<boost/serialization/string.hpp>
#include	<boost/serialization/vector.hpp>

#include	"version.h"
#include	"setup.h"
#include	"util.h"
#include	"config.h"

struct config;

struct stats_t {
	stats_t() 
		: hostname(get_host_name())
		, version(SB_VERSION)
		, startup(std::time(0))
		, nrequests(0)
		, nnewprocs(0)
	{
	}

	std::string hostname;
	std::string version;
	unsigned int startup;
	sw_atomic_t nrequests;
	sw_atomic_t nfailed;
	sw_atomic_t nnewprocs;
	sw_atomic_t nreaped;

	template<class Archive>
	void save(Archive &ar, const unsigned int version) const {
		uint32_t n;
		ar & hostname;
		ar & this->version;
		ar & startup;
		
		n = nrequests; ar & n;
		n = nfailed; ar & n;
		n = nnewprocs; ar & n;
		n = nreaped; ar & n;
	}

	template<class Archive>
	void load(Archive &ar, const unsigned int version) {
		uint32_t n;
		ar & hostname;
		ar & this->version;
		ar & startup;
		ar & n; nrequests = n;
		ar & n; nfailed = n;
		ar & n; nnewprocs = n;
		ar & n; nreaped = n;
	}
	
	BOOST_SERIALIZATION_SPLIT_MEMBER()
};

extern stats_t stats;

struct stats_thread {
	stats_thread(int);
	~stats_thread();

	void start();

	static void accept_handler(int, configp const &);

private:
	int fd_;
};

BOOST_CLASS_TRACKING(stats_t, boost::serialization::track_never);

#endif	/* !STATUS_THREAD_H */
