/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: switchboard.c 35 2008-08-20 06:02:49Z river $ */

#include	<sys/types.h>
#include	<sys/wait.h>

#include	<fcntl.h>
#include	<stdio.h>
#include	<unistd.h>
#include	<syslog.h>
#include	<stdlib.h>
#include	<string.h>

#include	"version.h"
#include	"setup.h"

#define BINPATH PREFIX "/lib/switchboard/switchboard-bin"

int dflag;

int
main(argc, argv)
	char **argv;
{
	int c;
	while ((c = getopt(argc, argv, "Vd")) != -1) {
		switch (c) {
		case 'V':
			printf("switchboard version %s (%s)\n", SB_VERSION, SB_CODENAME);
			printf("Copyright (c) River Tarnell <river@wikimedia.org>\n");
			return 0;

		case 'd':
			dflag++;
			break;

		default:
			return 1;
		}
	}

	fprintf(stderr, "switchboard %s (%s) starting...\n",
			SB_VERSION, SB_CODENAME);

	openlog("switchboard", LOG_PID, LOG_DAEMON);

	if (!dflag) {
		switch (fork()) {
		case -1:
			syslog(LOG_CRIT, "cannot fork: %m");
			return 1;
		case 0:
			break;
		default:
			return 0;
		}

		setsid();
		chdir("/");
		int fd;
		if ((fd = open("/dev/null", O_RDWR, 0)) != -1) {
			dup2(fd, STDIN_FILENO);
			dup2(fd, STDOUT_FILENO);
			dup2(fd, STDERR_FILENO);
			if (fd > STDERR_FILENO)
				close(fd);
		}
	}

	char *args[10];
	int i = 0;
	memset(args, 0, sizeof(args));

	args[i++] = BINPATH;
	args[i++] = "-d";
	if (!dflag)
		args[i++] = "-S";

	for (;;) {
		pid_t pid;
		switch (pid = fork()) {
		case -1:
			syslog(LOG_CRIT, "cannot fork: %m");
			exit(1);

		case 0:
			execv(BINPATH, args);
			_exit(254);

		default:
			break;
		}

		int stat;
		waitpid(pid, &stat, 0);

		if (WIFEXITED(stat)) {
			syslog(LOG_INFO, "child %d exited with status %d",
					pid, WEXITSTATUS(stat));
			return 0;
		}

		if (WIFSIGNALED(stat)) {
			if (WTERMSIG(stat) == SIGUSR1)
				return 0;

			syslog(LOG_INFO, "child %d exited with signal %d (%s)",
					pid, WTERMSIG(stat), strsignal(WTERMSIG(stat)));
		}

		sleep(1);
	}
}
