/* Copyright (c) 2008 River Tarnell <river@wikimedia.org>. */
/*
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely. This software is provided 'as-is', without any express or implied
 * warranty.
 */
/* $Id: util.cc 53 2008-08-25 21:27:34Z river $ */

#include	<sys/types.h>
#include	<sys/time.h>
#include	<sys/socket.h>
#include	<unistd.h>
#include	<poll.h>

#include	<boost/format.hpp>

#include	<iostream>
#include	<cerrno>
#include	<cstdlib>
#include	<cstring>
#include	<string>
#include	<vector>

#include	"util.h"

ssize_t
timed_read(int fd, void *buf, std::size_t nbytes, int timeout)
{
	if (timeout > -1) {
		ssize_t n = recv(fd, buf, nbytes, MSG_DONTWAIT);
		if (n >= 0 || (n == -1 && errno != EWOULDBLOCK))
			return n;

		struct pollfd pfd;
		std::memset(&pfd, 0, sizeof(pfd));
		pfd.fd = fd;
		pfd.events = POLLIN;
		int i = poll(&pfd, 1, timeout * 1000);
		if (i == -1)
			return -1;
		if (i == 0 || !pfd.revents) {
			errno = ETIMEDOUT;
			return -1;
		}
	}

	errno = 0;
	return read(fd, buf, nbytes);
}

ssize_t
timed_write(int fd, void const *buf, std::size_t nbytes, int timeout)
{
	if (timeout > -1) {
		struct pollfd pfd;
		std::memset(&pfd, 0, sizeof(pfd));
		pfd.fd = fd;
		pfd.events = POLLOUT;
		int i = poll(&pfd, 1, timeout * 1000);
		if (i == -1)
			return -1;
		if (i == 0 || !pfd.revents) {
			errno = ETIMEDOUT;
			return -1;
		}
	}

	errno = 0;
	return write(fd, buf, nbytes);
}

std::string
htmlescape(std::string const &src)
{
	std::string result;

	for (int i = 0, end = src.size(); i != end; ++i) {
		switch (src[i]) {
		case '&':
			result += "&amp;";
			break;
		case '<':
			result += "&lt;";
			break;
		case '>':
			result += "&gt;";
			break;
		case '"':
			result += "&quot;";
			break;
		case '\'':
			result += "&apos;";
			break;
		default:
			result += src[i];
			break;
		}
	}

	return result;
}

std::string
get_host_name()
{
	int len = sysconf(_SC_HOST_NAME_MAX);
	std::vector<char> buf(len + 1);
	gethostname(&buf[0], buf.size() - 1);
	return std::string(&buf[0]);
}

